function varargout = calcstrain(coord, d, sigs)
% function [m, msigs, prin, ellipse] = calstrain(coord, d, sigs)
% MATLAB code to calculate instantaneous displacement, rotation, and
% strain from a set of 3 non-colinear GPS stations.  Solution based on the
% method presented in Savage et al., 2001, modified for a flat earth.
%
% Inputs:  All data are sorted by row for each of the input stations.
% coord is position data lon,lat (decimal degrees) in 2 columns. d are 
% observed station velocities vx, vy (m/yr) in 2 columns. sigs are 
% uncertainty estimates ex, ey (m/yr) in 2 columns.
% Note: PBO velocity data are in mm/yr so convert units before defining
% variables.
%
% Outputs: m - model parameters translation in x,y (m/yr), rotation
% rate (radians/yr), and strain tensor components (exx, exy , eyy) in utm
% coordiante system. msigs - uncertainty in these parameters,
% and prin - the two principal strains and their associated axes 
% in the format: principal strain, axial vector (x,y) columns 1:3
% repeated for second principal strain in columns 4:6.
%
% Alternatively, if a single output argument O is specified, a structure 
% O containing fields identical to those returned in the Excel spreadsheet
% wll be output.  The field names are:
%   
%   transE - east translation rate
%   transN - north translation rate
%   transEsig - east translation rate uncertainty
%   transNsig - north translation rate uncertainty
%   transAz - translation vector azimuth
%   transSpeed - translation vector magnitude
%   rotDeg - rotation rate (degrees/yr)
%   rotDegSig - rotation rate uncertainty (degrees/yr)
%   rotnRad - rotation rate (nanoradians/yr)
%   rotnRadSig - rotation rate uncertainty (nanoradians/yr)
%   rotDir - rotation direction 
%   e1H - max. horizontal extension (nano-strain/yr)
%   S1Haz - azimuth of max. horizontal extension
%   e2H - min. horizontal extension (nano-strain/yr)
%   S1Haz - azimuth of min. horizontal extension
%   maxShear = maximum horizontal shear strain (nano-strain/yr)
%   areaStrain = are strain (nano-strain/yr)
%
% Note that this code will not work properly for stations spanning two UTM
% zones.
%
% Written by Phil Resor, January 7, 2013 as part of the UNAVCO sponsored
% education & outreach project "Infinitesimal strain analysis using GPS 
% data: Module for structural geology or geophysics course"
% http://www.unavco.org/edu_outreach/resources/gps-strain/majors-gps-strain
% /majors-gps-strain.html

% isolate coordinate data and convert to UTM
[x,y]=wgs2utm(coord(:,2), coord(:,1));
X=[x,y];
   
% calculate mean x and y positions
meanX = 1/3 * (sum(X(:,1)));
meanY = 1/3 * (sum(X(:,2)));

centroid = [meanX,meanY];

% remove mean position to create local coordinates
X=[X(:,1)-meanX, X(:,2)-meanY];

% get velocity data and reformat into column vector d
d = d'; % velocities by row then column
d = d(:); % make column vector

% get error data and reformat into covariance matrix covd
sigs = sigs'; % uncertainties by row then column
sigs=sigs(:); % make column vector

% create diagonal covariance matrix (assuming all errors independent)
covd = diag(1./sigs.^2,0); 

% set up coefficient matrix
G = [1, 0, -X(1,2), X(1,1), X(1,2), 0;...
     0, 1, X(1,1), 0, X(1,1), X(1,2);...
     1, 0, -X(2,2), X(2,1), X(2,2), 0;...
     0, 1, X(2,1), 0, X(2,1), X(2,2);...
     1, 0, -X(3,2), X(3,1), X(3,2), 0;...
     0, 1, X(3,1), 0, X(3,1), X(3,2)];

% solve for model paramaters (tx, ty, w, exx, exy, eyy)
m=G\d;

% assemble strain tensor
E=[m(4) m(5); m(5) m(6)];

% find principal strains and axes
[paxes, pvals]=eig(E);
prin=[pvals(1), paxes(:,1)', pvals(4), paxes(:,2)'];

% estimate model error
covm=inv(G'*covd*G);

% extract diagonal elements adn take square root to estimate error in model
% paramaters
msigs=diag(covm,0).^(0.5);

% plot ellipse
v1 = [-1 0];
v2 = [prin(5),prin(6)];

ang = (atan2(abs(v1(1)*v2(2)-v1(2)*v2(1)),v1(1)*v2(1)+v1(2)*v2(2)))*180/pi;


if v2(1)<0 && v2(2)<0
    ang = -ang;
end

[E1 E2] = calculateEllipse(centroid(1), centroid(2), 25*(500 + prin(4)), 25*(500 + prin(1)), ang);
figure(1)
ellipse = plot(E1,E2,'r-','LineWidth',1);

if nargout == 1
	% If only one output argument is specified, 
	% prepare structure with fields identical to Excel results
	
	% Translation rate vector
	o.transE = m(1);
	o.transN = m(2);
	o.transEsig = msigs(1);
	o.transNsig = msigs(2);
	o.transAz = 90-180/pi*atan2(m(2), m(1));
	o.transAz(o.transAz < 0) = o.transAz(o.transAz < 0) + 360;
	o.transSpeed = sqrt(m(1).^2 + m(2).^2);
	
	% Rotation rate
	o.rotDeg = 180/pi*m(3);
	o.rotDegSig = 180/pi*msigs(3);
	o.rotnRad = 1e9*m(3);
	o.rotnRadSig = 1e9*msigs(3);
	if o.rotDeg < 0
		o.rotDir = 'Clockwise';
	else
		o.rotDir = 'Counterclockwise';
	end
	
	% Principal strain rates
	prinaz = 90-180/pi*atan2(paxes(2, :)', paxes(1, :)');
	prinaz(prinaz < 0) = prinaz(prinaz < 0) + 360;
	o.e1H = 1e9*pvals(4);
	o.S1Haz = prinaz(2);
	o.e2H = 1e9*pvals(1);
	o.S2Haz = prinaz(1);
	
	% Other strain rates
	o.maxShear = 1e9*2*sqrt(((m(4)-m(6))/2).^2 + m(5).^2);
	o.areaStrain = 1e9*sum(pvals([1 4]));
	
	varargout{1} = o;

elseif nargout == 4

   % If three output arguments are specified, they are as described in help
   varargout{1} = m;
   varargout{2} = msigs;
   varargout{3} = prin;
   varargout{4} = ellipse;

end

    function  [x,y,utmzone] = wgs2utm(Lat,Lon)
        % modified from code created by Alexandre Schimel, MetOcean Solutions Ltd, 
        % New Plymouth, New Zealand, available for download on MatlabCentral
        % simplified here to automatically choose UTM zone

        % coordinates in radians
        lat = Lat.*pi./180;
        lon = Lon.*pi./180;

        % WGS84 parameters
        a = 6378137;           %semi-major axis
        b = 6356752.314245;    %semi-minor axis
        e = sqrt(1-(b./a).^2); % eccentricity

        Lon0 = floor(Lon./6).*6+3; % reference longitude in degrees
        lon0 = Lon0.*pi./180;      % in radians
        k0 = 0.9996;               % scale on central meridian

        FE = 500000;              % false easting
        FN = (Lat < 0).*10000000; % false northing

        % Equations parameters
        eps = e.^2./(1-e.^2);  % e prime square
        % N: radius of curvature of the earth perpendicular to meridian plane
        % Also, distance from point to polar axis
        N = a./sqrt(1-e.^2.*sin(lat).^2); 
        T = tan(lat).^2;                
        C = ((e.^2)./(1-e.^2)).*(cos(lat)).^2;
        A = (lon-lon0).*cos(lat);                            
        % M: true distance along the central meridian from the equator to lat
        M = a.*(  ( 1 - e.^2./4 - 3.*e.^4./64 - 5.*e.^6./256 )  .* lat         ...
                 -( 3.*e.^2./8 + 3.*e.^4./32 + 45.*e.^6./1024 ) .* sin(2.*lat) ...
                 +( 15.*e.^4./256 + 45.*e.^6./1024 )            .* sin(4.*lat) ...
                 -(35.*e.^6./3072 )                             .* sin(6.*lat) );

        % easting
        x = FE + k0.*N.*(                                  A       ...
                         + (1-T+C)                      .* A.^3./6 ...
                         + (5-18.*T+T.^2+72.*C-58.*eps) .* A.^5./120 );

        % northing 
        y = FN + k0.*M + k0.*N.*tan(lat).*(                                     A.^2./2  ...
                                           + (5-T+9.*C+4.*C.^2)              .* A.^4./24 ...
                                           + (61-58.*T+T.^2+600.*C-330.*eps) .* A.^6./720 );

        % zone
        utmzone = floor(Lon0./6)+31;


function [X Y] = calculateEllipse(x, y, a, b, angle, steps)
    %# This functions returns points to draw an ellipse
    %#
    %#  @param x     X coordinate
    %#  @param y     Y coordinate
    %#  @param a     Semimajor axis
    %#  @param b     Semiminor axis
    %#  @param angle Angle of the ellipse (in degrees)
    %#

    error(nargchk(5, 6, nargin));
    if nargin<6, steps = 36; end

    beta = -angle * (pi / 180);
    sinbeta = sin(beta);
    cosbeta = cos(beta);

    alpha = linspace(0, 360, steps)' .* (pi / 180);
    sinalpha = sin(alpha);
    cosalpha = cos(alpha);

    X = x + (a * cosalpha * cosbeta - b * sinalpha * sinbeta);
    Y = y + (a * cosalpha * sinbeta + b * sinalpha * cosbeta);

    if nargout==1, X = [X Y]; end

